package cn.com.qmth.stmms.admin.dto;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;

public class SubjectQuestionDTO {

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    private int examId;

    private String subjectCode;

    private List<ExamQuestion> list;

    private double totalScore;

    private boolean objective;

    private Map<String, List<ExamQuestion>> paperTypeList;

    private Map<Integer, MarkGroup> groups;

    public SubjectQuestionDTO(int examId, String subjectCode, boolean objective) {
        this.examId = examId;
        this.subjectCode = subjectCode;
        this.list = new LinkedList<ExamQuestion>();
        this.paperTypeList = new HashMap<>();
        this.groups = new HashMap<Integer, MarkGroup>();
        this.totalScore = 0d;
        this.objective = objective;
    }

    public void addQuestion(ExamQuestion question) {
        if (question != null && question.getTotalScore() != null) {
            question.setExamId(examId);
            list.add(question);

            if (objective) {
                String paperType = question.getPaperType();
                if (paperType == null) {
                    paperType = NULL_PAPER_TYPE_PLACEHOLDER;
                }
                List<ExamQuestion> list = paperTypeList.get(paperType);
                if (list == null) {
                    list = new ArrayList<>();
                    paperTypeList.put(paperType, list);
                }
                list.add(question);
            } else {
                MarkGroup group = groups.get(question.getMainNumber());
                if (group == null) {
                    group = new MarkGroup(examId, subjectCode, question.getMainNumber(), question.getMainTitle(),
                            PictureConfigItem.parse(question.getPicList()), 0d, question.getDoubleRate(),
                            question.getArbitrateThreshold(), question.getScorePolicy(), question.getMarkMode(),
                            question.getTrialCount(), false);
                    group.setQuestionList(new LinkedList<ExamQuestion>());
                    groups.put(question.getMainNumber(), group);
                }
                group.getQuestionList().add(question);
                group.setTotalScore(group.getTotalScore() + question.getTotalScore());
                totalScore += question.getTotalScore();
            }
        }
    }

    public List<ExamQuestion> getQuestionList() {
        return list;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public double getTotalScore() {
        return totalScore;
    }

    public Map<Integer, MarkGroup> getGroups() {
        return groups;
    }

    public boolean validate(List<String> error) {
        if (list.size() > 0) {
            Set<String> numberSet = new HashSet<String>();
            Map<Integer, String> titleMap = new HashMap<Integer, String>();
            try {
                // 基本字段合法性校验
                for (ExamQuestion question : list) {
                    if (StringUtils.isBlank(question.getMainTitle())) {
                        error.add("[" + subjectCode + "] 有大题名称为空的记录");
                        return false;
                    }
                    if (question.getMainTitle().length() > 30) {
                        error.add("[" + subjectCode + "] 有大题名称超过30个字的记录");
                        return false;
                    }
                    if (question.getMainNumber() == null || question.getSubNumber() == null) {
                        error.add("[" + subjectCode + "] 有题号为空的记录");
                        return false;
                    }
                    if (question.getTotalScore() == null) {
                        error.add("[" + subjectCode + "] 有满分为空的记录");
                        return false;
                    }
                    if (objective && StringUtils.isBlank(question.getAnswer())) {
                        error.add("[" + subjectCode + "] 有答案为空的记录");
                        return false;
                    }

                    String title = titleMap.get(question.getMainNumber());
                    if (title != null && !title.equals(question.getMainTitle())) {
                        error.add("[" + subjectCode + "] 大题号" + question.getMainNumber() + "有名称不一致的记录");
                        return false;
                    } else {
                        titleMap.put(question.getMainNumber(), question.getMainTitle());
                    }

                    String number = question.getQuestionNumber();
                    if (objective) {
                        number = StringUtils.trimToEmpty(question.getPaperType()) + " " + number;
                    }
                    if (numberSet.contains(number)) {
                        error.add("[" + subjectCode + "] 题号" + number + "有重复记录");
                        return false;
                    } else {
                        numberSet.add(number);
                    }
                }

                // 客观题时额外校验不同试卷类型的总分需要一致
                if (objective) {
                    totalScore = -1;
                    for (Entry<String, List<ExamQuestion>> entry : paperTypeList.entrySet()) {
                        double score = 0;
                        for (ExamQuestion question : entry.getValue()) {
                            score += question.getTotalScore();
                        }
                        if (totalScore == -1) {
                            totalScore = score;
                        } else if (totalScore != score) {
                            error.add("[" + subjectCode + "] 多个试卷类型总分不一致");
                            return false;
                        }
                    }
                }
                return true;
            } finally {
                numberSet.clear();
                titleMap.clear();
            }
        } else {
            return false;
        }
    }
}
